import os
import json
import argparse

from datetime import datetime

"""
AUTHOR:
SARAN GUNASEKARAN

EMAIL:
saran.gunasekara@tigeranalytics.com/
saran.gunasekaran@effem.com
"""


class DirectoryAndJSONGenerator:
    def __init__(self, layer_name, project_name, config_dict):
        self.layer_name = layer_name
        self.project_name = project_name
        self.config_dict = config_dict
        self.input_dict = {}
        self.output_dict = {}
        self.deequ_dict = {}
        self.project_config_dict = {}
        self.subdirectories = ["settings", "dags", "ddls", "transform"]

    def create_directories(self):
        # Define the base directory
        base_dir = os.path.join(self.layer_name, self.project_name)

        # Create the base directory
        os.makedirs(base_dir, exist_ok=True)

        # Create the subdirectories
        for subdir in self.subdirectories:
            os.makedirs(os.path.join(base_dir, subdir), exist_ok=True)

    def generate_input_json(self):
        for process_name, inputs in self.config_dict["PROCESS_NAME"].items():
            input_list = []
            for input_data in inputs:
                input_dict_entry = {
                    "dataframe_name": input_data[0].split("/")[-1].split(".")[0],
                    "type_of_input": "lake",
                    "input": input_data[0],
                    "input_delimiter": ",",
                    "input_file_type": input_data[1],
                    "start_date": "na",
                    "input_condition": None
                }
                input_list.append(input_dict_entry)
            self.input_dict[process_name] = input_list

        # Write input JSON to settings.json
        input_settings_file_path = os.path.join(self.layer_name, self.project_name, 'settings', 'input.json')
        with open(input_settings_file_path, 'w') as json_file:
            json.dump(self.input_dict, json_file, ensure_ascii=False, indent=2)

    def generate_output_json(self):
        today = datetime.now().strftime("%Y-%m-%d")
        for process_name, inputs in self.config_dict["PROCESS_NAME"].items():
            output_list = []
            for input_data in inputs:
                output_dict_entry = {
                    "table_name": input_data[0].split("/")[-1].split(".")[0],
                    "partition": "row_valid_to_ts",
                    "output_name": input_data[0].split("/")[-1].split(".")[0],
                    "insert_condition": None,
                    "driver_for_process_date": today,
                    "driver_for_process_edate": today,
                    "driver_cadence": "daily",
                    "driver_input_location": input_data[0],
                    "driver_date_format": "yyyy-mm-dd",
                    "type_of_write": "delta_append",
                    "write_database": "raw",
                    "primary_keys": None,
                    "surrogate_key": None,
                    "stamp": "date",
                    "folder": f"raw/{self.project_name}/"
                }
                output_list.append(output_dict_entry)
            self.output_dict[process_name] = output_list

        # Write output JSON to output.json
        output_settings_file_path = os.path.join(self.layer_name, self.project_name, 'settings', 'output.json')
        with open(output_settings_file_path, 'w') as json_file:
            json.dump(self.output_dict, json_file, ensure_ascii=False, indent=2)

    def generate_deequ_json(self):
        for process_name, inputs in self.config_dict["PROCESS_NAME"].items():
            deequ_dict_entry = {}
            for input_data in inputs:
                table_name = input_data[0].split("/")[-1].split(".")[0]
                deequ_dict_entry[table_name] = {
                    "complete": [],
                    "unique": [],
                    "contained_in": {},
                    "non_negative": [],
                    "size": 1,
                    "has_min": {},
                    "mean": {},
                    "sum": {},
                    "standard_deviation": {},
                    "type": "hive",
                    "layer": "raw"
                }
            self.deequ_dict[process_name] = deequ_dict_entry

        # Write deequ JSON to deequ.json
        deequ_settings_file_path = os.path.join(self.layer_name, self.project_name, 'settings', 'deequ.json')
        with open(deequ_settings_file_path, 'w') as json_file:
            json.dump(self.deequ_dict, json_file, ensure_ascii=False, indent=2)

    def generate_project_config_json(self):
        project_config = {
            "LOCAL": {
                "environment": "LOCAL",
                "input": f"src/common_data_layer/{self.layer_name}/settings/{self.project_name}/input.json",
                "output": f"src/common_data_layer/{self.layer_name}/settings/{self.project_name}/output.json",
                "deequ": f"src/common_data_layer/{self.layer_name}/settings/{self.project_name}/deequ.json",
                "dags": f"src/common_data_layer/{self.layer_name}/settings/{self.project_name}/dags.json",
                "read_location": "C:/spark-warehouse/",
                "write_location": "C:/spark-warehouse/output/",
                "logging_location": "C:/spark-warehouse/logs/",
                "project_mailing_list": [
                    "saran.gunasekaran@effem.com"
                ],
                "data_catalog_config": f"/data_catalog/{self.project_name}/data_catalog.yml",
                "spark_home": "C:/spark-3.3.2-bin-hadoop3",
                "hadoop_home": "C:/hadoop-3.2.2/",
                "logic_app_url": "https://prod-25.eastus2.logic.azure.com:443/workflows/c7f54865034b465d96f3818518a7416b/triggers/manual/paths/invoke?api-version=2016-10-01&sp=%2Ftriggers%2Fmanual%2Frun&sv=1.0&sig=aiEdVb_7BGbSxQz5021DDgxYTykzy7nGbPX1wGRcfaE"
            },
            "DEV": {
                "environment": "DEV",
                "input": f"{{}}/src/common_data_layer/{self.layer_name}/settings/{self.project_name}/input.json",
                "output": f"{{}}/src/common_data_layer/{self.layer_name}/settings/{self.project_name}/output.json",
                "deequ": f"{{}}/src/common_data_layer/{self.layer_name}/settings/{self.project_name}/deequ.json",
                "dags": f"{{}}/src/common_data_layer/{self.layer_name}/settings/{self.project_name}/dags.json",
                "read_location": "dbfs:/mnt",
                "write_location": "dbfs:/mnt/dev/RAW/",
                "checkpoint_location": "/spark-warehouse/checkpoints",
                "schema_location": "/stream_testing/json_schema",
                "project_mailing_list": [
                    "saran.gunasekaran@effem.com"
                ],
                "data_catalog_config": f"/data_catalog/{self.project_name}/data_catalog.yml",
                "logic_app_url": "https://prod-25.eastus2.logic.azure.com:443/workflows/c7f54865034b465d96f3818518a7416b/triggers/manual/paths/invoke?api-version=2016-10-01&sp=%2Ftriggers%2Fmanual%2Frun&sv=1.0&sig=aiEdVb_7BGbSxQz5021DDgxYTykzy7nGbPX1wGRcfaE",
                "notification_type": "TEAMS"
            },
            "PROD": {
                "environment": "PROD",
                "input": f"{{}}/src/common_data_layer/{self.layer_name}/settings/{self.project_name}/input.json",
                "output": f"{{}}/src/common_data_layer/{self.layer_name}/settings/{self.project_name}/output.json",
                "deequ": f"{{}}/src/common_data_layer/{self.layer_name}/settings/{self.project_name}/deequ.json",
                "dags": f"{{}}/src/common_data_layer/{self.layer_name}/settings/{self.project_name}/dags.json",
                "read_location": "dbfs:/mnt/",
                "write_location": "dbfs:/mnt/",
                "checkpoint_location": "/spark-warehouse/checkpoints",
                "schema_location": "/stream_testing/json_schema",
                "project_mailing_list": [
                    "saran.gunasekaran@effem.com"
                ],
                "data_catalog_config": f"/data_catalog/{self.project_name}/data_catalog.yml",
                "logic_app_url": None,
                "notification_type": "TEAMS"
            }
        }
        # Write project configuration JSON to <project_name>.json
        project_config_file_path = os.path.join(self.layer_name, self.project_name, 'settings',
                                                self.project_name + ".json")
        with open(project_config_file_path, 'w') as json_file:
            json.dump(project_config, json_file, ensure_ascii=False, indent=2)

    def generate(self):
        self.create_directories()
        self.generate_input_json()
        self.generate_output_json()
        self.generate_deequ_json()
        self.generate_project_config_json()


def main():
    parser = argparse.ArgumentParser(description="Generate project package")
    parser.add_argument("layer_name", help="Name of the layer")
    parser.add_argument("project_name", help="Name of the project")
    parser.add_argument("config_dict", help=f"Provide input paths for each process in this format:")
    args = parser.parse_args()

    # Parse the JSON config file
    with open(args.config_dict, 'r') as file:
        config_dict = json.load(file)

    # Instantiate the class and generate directories, input JSON, output JSON, deequ JSON, and project config JSON
    directory_and_json_generator = DirectoryAndJSONGenerator(args.layer_name, args.project_name, config_dict)
    directory_and_json_generator.generate()


if __name__ == "__main__":
    main()
